﻿using System;
using System.Security.Cryptography;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

/*
 * Supreme Commander 2 Mod Manager
 * Author: Kevin Scholl (AKA: Cerus)
 * Description: An external mod handling utility for Supreme Commander 2, allows the user to add and remove mods from the gamedata folder of Supreme Commander 2 without
 * destroying the default file structure in the process, or requiring any knowledge of standard mod installation procedure.
 * Allows advanced users to perform conflict detection to assist with finding compatability problems.
 * Project Started 02/02/2011
 * Version 1.0 (05/24/2011)
*/
namespace SupCom2_Mod_Manager
{
    public partial class frmMain : Form
    {
        public static double Version = 1.02;
        List<SC2MMData.Mod> Mods = new List<SC2MMData.Mod> { };

        public void ToLog(string instring)
        {
            if (SettingsData.EnableLogging)
            {
                StreamWriter sw = new StreamWriter(SC2MMData.GetAppDataPath() + "log.txt", true);
                sw.WriteLine(DateTime.Now.ToString() + " || " + instring);
                sw.Close();

                
            }
            else
            {
                //Do nothing for now.  Maybe a seperate form for real-time logging as an option later.
            }
        }
        
        public frmMain()
        {
            InitializeComponent();
        }

        private void LoadSettings()
        {
            if (File.Exists(SC2MMData.GetAppDataPath() + "settings.ini"))
            {

                string f_settings = SC2MMData.GetAppDataPath() + "settings.ini";
                FileInfo modfile = new FileInfo(SC2MMData.GetAppDataPath() + "SC2MM.dat");

                string settingsVersion = SettingsData.ReadCFG(f_settings, "$SC2MMVersion ");
                if (settingsVersion.Contains("No Data for ") == false)
                {
                    if (Version > double.Parse(settingsVersion))
                    {
                        MessageBox.Show("Configuration data from SC2MM [v" + SettingsData.ReadCFG(f_settings, "$SC2MMVersion ") + "] detected.\n");

                        //Future version migration logic goes here.



                        //Future version migration logic goes here.

                        ReadSettings(f_settings);
                        SaveSettings();

                        MessageBox.Show("Updated to [v" + Version + "]");
                        return;
                    }
                }
                else 
                {
                    if (modfile.Length != 0)
                    {
                        MessageBox.Show("Cannot migrate mods, run last used version of SC2MM and remove all loaded mods.");
                        SettingsData.AbortSaveLoad = true;
                        Application.Exit();
                        return;
                    }
                    else
                    {
                        BadSettings();
                        MessageBox.Show("Updated to [v" + Version + "]");
                    }

                }

                ReadSettings(f_settings);
                
                if (SettingsData.ModDataFolder.Contains("No Data for "))
                {
                    BadSettings();
                    return;
                }

                if (!Directory.Exists(SettingsData.SC2Folder))
                {
                    SettingsData.SC2Folder = SC2MMData.GetSupCom2Path();
                }
                ToLog("Settings loaded");
            }
            else
            {
                SettingsData.NewSettings();
                ToLog("Settings file missing, new file created with defaults.");
                SaveSettings();
            }
        }

        private void ReadSettings(string f_settings) 
        {
            SettingsData.ModDataFolder = SettingsData.ReadCFG(f_settings, "$ModDataFolder ");
            SettingsData.SC2Folder = SettingsData.ReadCFG(f_settings, "$SC2Folder ");
            SettingsData.PersistentLoading = bool.Parse(SettingsData.ReadCFG(f_settings, "$PersistentLoading "));
            SettingsData.EnableLogging = bool.Parse(SettingsData.ReadCFG(f_settings, "$EnableLogging "));
            return;
        }

        private void BadSettings()
        {
            MessageBox.Show("Settings file recreated.");
            SettingsData.NewSettings();
            SaveSettings();
        }

        private void SaveSettings()
        {
            if (!SettingsData.SaveBlock)
            {
                StreamWriter sw = new StreamWriter(SC2MMData.GetAppDataPath() + "settings.ini");
                sw.WriteLine("$SC2MMVersion <<" + Version + ">>");
                sw.WriteLine("$ModDataFolder <<" + SettingsData.ModDataFolder + ">>");
                sw.WriteLine("$SC2Folder <<" + SettingsData.SC2Folder + ">>");
                sw.WriteLine("$PersistentLoading <<" + SettingsData.PersistentLoading.ToString() + ">>");
                sw.WriteLine("$EnableLogging <<" + SettingsData.EnableLogging.ToString() + ">>");
                sw.Close();
                ToLog("Settings saved");
            }
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            bool ValidInstance;
            System.Threading.Mutex MX = new System.Threading.Mutex(true, "SingleSC2MM", out ValidInstance);
            if (!ValidInstance)
            {
                MessageBox.Show("Another instance of this program is already running.");
                ToLog("Program was already running, killing duplicate...");
                Application.Exit();
                return;
            }
            GC.KeepAlive(MX);

            if (Directory.Exists(SC2MMData.GetAppDataPath())) //Retrieve the saved mods list from the file and reload the contents into our list
            {
                SettingsData.AbortSaveLoad = false;
                LoadSettings();
                LoadModLibrary();
            }
            else
            {
                Directory.CreateDirectory(SC2MMData.GetAppDataPath());
                //MessageBox.Show("The application folder was missing and has been (re)created.", "Missing App Folder");
                SettingsData.NewSettings();
                SaveSettings();
            }

            string[] folders = new string[] {
                SC2MMData.GetModsPath(),
                SC2MMData.GetTempPath(),
                SC2MMData.GetSCDPath(),
                SC2MMData.GetDescPath()
            };

            foreach (string path in folders)
            {
                if (!Directory.Exists(path) & !SettingsData.SaveBlock)
                    Directory.CreateDirectory(path);
            }
        }

        private void frmMain_FormClosed(object sender, FormClosedEventArgs e)  //Start saving mods and clean up the temp folder
        {
            SaveModLibrary();
            if (!SettingsData.PersistentLoading)
            {
                foreach (SC2MMData.Mod u_mod in Mods)
                {
                    if (u_mod.Enabled)
                    {
                        ModLoader(u_mod, false);
                    }

                }
            }
        }



        private void SaveModLibrary()  // Saves contents of mod list to file
        {
            if (SettingsData.AbortSaveLoad)
                return;

            StreamWriter sw = new StreamWriter(SC2MMData.GetAppDataPath() + "SC2MM.dat");
            foreach (SC2MMData.Mod m_mod in Mods)
            {
                m_mod.Priority = FindIndexByID(m_mod.ID); //Sets priority by current index in Mods list
                sw.Write(ApDelim(m_mod.ID)); //0
                sw.Write(ApDelim(m_mod.ModName)); //1
                sw.Write(ApDelim(m_mod.Author)); //2
                sw.Write(ApDelim(m_mod.ShortDesc)); //3
                sw.Write(ApDelim(m_mod.LongDesc)); //4
                sw.Write(ApDelim(m_mod.Priority.ToString())); //5
                sw.Write(ApDelim(m_mod.DirRef)); //6
                sw.Write(ApDelim(m_mod.FileRef)); //7
                sw.Write(ApDelim(m_mod.Version.ToString())); //8
                sw.Write(ApDelim(m_mod.Enabled.ToString())); //9
                sw.Write(ApDelim(m_mod.MultiMod.ToString())); //10
                sw.Write(ApDelim(m_mod.ModURL)); //11
                sw.Write(ApDelim(m_mod.ReqModName)); //12
                sw.Write(ApDelim(m_mod.ReqModID)); //13
                sw.Write(ApDelim(m_mod.ReqModVer)); //14
                sw.Write(ApDelim(m_mod.Conflicts)); //15
                sw.Write("\n");
            }
            sw.Close();
            ToLog("Mod library saved to " + SC2MMData.GetAppDataPath() + "SC2MM.dat");
        }

        private string ApDelim(string t_snip) //Append with delimiter to string
        {
            string delim = "||";
            string appended = t_snip + delim;
            return appended;
        }

        private void LoadModLibrary()
        {
            if (SettingsData.AbortSaveLoad)
                return;

            string infopath = SC2MMData.GetAppDataPath() + "SC2MM.dat";
            if (File.Exists(infopath))
            {
                string[] s_line = { "||" };
                string[] rawText;
                List<string> line = new List<string> { };
                StreamReader sr = new StreamReader(infopath);
                while (!sr.EndOfStream)
                {
                    rawText = sr.ReadLine().Split(s_line, StringSplitOptions.None);
                    line.Clear();
                    foreach (string str in rawText)
                    {
                        line.Add(str);
                    }

                    SC2MMData.Mod l_mod = new SC2MMData.Mod
                    {
                        ID = line[0],
                        ModName = line[1],
                        Author = line[2],
                        ShortDesc = line[3],
                        LongDesc = line[4],
                        Priority = int.Parse(line[5]),
                        DirRef = line[6],
                        FileRef = line[7],
                        Version = double.Parse(line[8]),
                        Enabled = bool.Parse(line[9]),
                        MultiMod = bool.Parse(line[10]),
                        ModURL = line[11],
                        ReqModName = line[12],
                        ReqModID = line[13],
                        ReqModVer = line[14],
                        Conflicts = line[15]
                    };

                    if (!PathIsCorrupt(l_mod.DirRef))
                    {
                        SettingsData.LoadingPhase = true;
                        Mods.Add(l_mod);
                        AuthorizeCheck = true;
                        AddToList(FindIndexByID(l_mod.ID));
                        AuthorizeCheck = false;
                        SettingsData.LoadingPhase = false;
                    }
                    else
                    {

                        if (MessageBox.Show("Mod: " + l_mod.ModName + " is missing mod folders. Remove orphaned folders?\n(Mod could not be loaded)", "Missing Folders", MessageBoxButtons.YesNo) == DialogResult.Yes)
                        {
                            string checkPath = SC2MMData.GetModsPath() + l_mod.DirRef;
                            if (Directory.Exists(checkPath))
                                Directory.Delete(checkPath, true);
                            checkPath = SC2MMData.GetDescPath() + l_mod.DirRef;
                            if (Directory.Exists(checkPath))
                                Directory.Delete(checkPath, true);
                            checkPath = SC2MMData.GetSCDPath() + l_mod.DirRef;
                            if (Directory.Exists(checkPath))
                                Directory.Delete(checkPath, true);
                            ToLog("Folder for mod " + l_mod.ModName + " (" + l_mod.ID + ") was missing; removed from library and folders cleaned");
                        }
                        else
                        {
                            MessageBox.Show("The orphaned folders are located under the desc,mod,or scd folders in\n" + SettingsData.ModDataFolder + "\n with the folder name of " + l_mod.DirRef);
                            ToLog("Folder for mod " + l_mod.ModName + " (" + l_mod.ID + ") was missing; removed from library, folders NOT cleaned");
                        }
                    }
                }
                sr.Close();
            }
        }

        private static bool PathIsCorrupt(string modpath)
    {
        string checkPath = SC2MMData.GetSCDPath() + modpath;
        if (!Directory.Exists(checkPath))
            return true;
        checkPath = SC2MMData.GetModsPath() + modpath;
        if (!Directory.Exists(checkPath))
            return true;
        checkPath = SC2MMData.GetDescPath() + modpath;
        if (!Directory.Exists(checkPath))
            return true;
    
        return false;
    }

        private void chklbModList_SelectedIndexChanged(object sender, EventArgs e) //Put the descriptive data into the labels and statusbar on a selection change
        {
            if (chklbModList.SelectedItem != null)
            {

                SC2MMData.Mod s_mod = Mods[chklbModList.SelectedIndex];
                string ld_file = SC2MMData.GetDescPath() + s_mod.DirRef + s_mod.LongDesc;
                StatusStrip.Text = " \"" + s_mod.ShortDesc + "\"" + " Made by: " + s_mod.Author + " [v" + s_mod.Version + "]";
                if (File.Exists(ld_file))
                {
                    if (ld_file.EndsWith(".rtf"))
                        txtLongDesc.LoadFile(ld_file);
                    else if (ld_file.EndsWith(".txt"))
                        txtLongDesc.Text = ReadLongDescriptionFromFile(ld_file);
                }
                else
                    txtLongDesc.Text = s_mod.ShortDesc;

                mnuForumThread.Enabled = (Mods[chklbModList.SelectedIndex].ModURL.Contains("No Data for") == false & Mods[chklbModList.SelectedIndex].ModURL != "");
                
            }
        }

        private string ReadLongDescriptionFromFile(string file)
        {
            string textFileContent = "";
            StreamReader sr = new StreamReader(file);
            textFileContent = sr.ReadToEnd();
            sr.Close();
            return textFileContent;
        }

        private void txtLongDesc_LinkClicked(object sender, LinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start(e.LinkText);
        }

        private void ClearLabels()
        {
            StatusStrip.Text = "";
            txtLongDesc.Text = "";
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e) //Exit from menu
        {
            Application.Exit();
        }

        private void btnAddMod_Click(object sender, EventArgs e) //Adds a mod via main form menu button
        {
            AddBrowse();
        }

        private void AddBrowse()  //Opens a file dialog box to select a new mod file.
        {
            if (filedialog.ShowDialog() == DialogResult.OK)
            {
                FileInfo modfile = new FileInfo(filedialog.FileName);
                AddToMods(modfile);
            }
        }

        private void AddToMods(FileInfo modfile)
        {

            SC2MMData.Mod newmod = new SC2MMData.Mod { };
            newmod = SC2MMData.Mod.AddMod(modfile);
            //MessageBox.Show(newmod.Patch.ToString());
            if (newmod.Priority >= 0 & !FindDupe(newmod))
            {
                    Mods.Add(newmod);
                    SC2MMData.Mod.FinalizeMod(modfile, newmod);
                    AddToList(FindIndexByID(newmod.ID));
                    ToLog(newmod.ModName + "(" + newmod.ID + ") v" + newmod.Version + " was successfully added.");
            }
            else if (newmod.Priority >= 0 & FindDupe(newmod) & newmod.Patch)
            {
                Mods[FindIndexByID(newmod.ID)] = SC2MMData.Mod.FinalizePatchMod(newmod, Mods[FindIndexByID(newmod.ID)]);
                RemoveFromList(FindIndexByID(newmod.ID));
                AddToList(FindIndexByID(newmod.ID));
                ToLog(newmod.ModURL + "(" + newmod.ID + ") patched to v" + newmod.Version);
            }
            else
            {
                if (newmod.Priority < 0)
                    MessageBox.Show("Invalid or missing configuration file.");
                else
                {
                    if (VersionDiff(newmod))
                    {
                        ModRemove(FindIndexByID(newmod.ID));
                        SC2MMData.Mod.AddMod(modfile);
                        Mods.Add(newmod);
                        SC2MMData.Mod.FinalizeMod(modfile, newmod);
                        AddToList(FindIndexByID(newmod.ID));
                        ToLog(newmod.ModName + "(" + newmod.ID + ") was updated to v" + newmod.Version);
                    }
                    else
                    {
                        MessageBox.Show("Selected mod conflicts with existing mod in library", "Loading failed");
                        ToLog(newmod.ModName + "(" + newmod.ID + ") Not added due to conflict");
                    }
                }
            }
        }

        

        private void AddToList(int m_index)
        {
            chklbModList.Items.Add(Mods[m_index].ModName, Mods[m_index].Enabled);
        }

        private void RemoveFromList(int m_index)
        {
            chklbModList.Items.RemoveAt(m_index);
        }


        private void WipeModFolders(SC2MMData.Mod d_mod) //Removes the folders associated with a mod
        {

            string[] folders = new string[] {
                SC2MMData.GetModsPath() + d_mod.DirRef,
                SC2MMData.GetTempPath() + d_mod.DirRef,
                SC2MMData.GetSCDPath() + d_mod.DirRef,
                SC2MMData.GetDescPath() + d_mod.DirRef
            };

            foreach (string path in folders)
            {
                if (Directory.Exists(path))
                    Directory.Delete(path,true);
            }

        }

        private void chklbModList_ItemCheck(object sender, ItemCheckEventArgs e) //Toggles enabled state of selected mod in Mods list by checkbox index on itemcheck event
        {

            if (!AuthorizeCheck)
            {
                e.NewValue = e.CurrentValue; //check state change was not through authorized actions
            }
            else
            {
                AuthorizeCheck = false;
                bool load_result = false;
                bool c_enabled = (e.NewValue == CheckState.Checked);
                bool PassReqCheck = true;
                bool PassConflictCheck = true;

                if (ModReqCheck(Mods[e.Index]))
                {
                    PassConflictCheck = ConflictCheck(e.Index);
                    if (c_enabled == false)
                    {
                        PassReqCheck = ReqModCheck(e.Index);
                    }
                    if (PassReqCheck & PassConflictCheck)
                    {
                        load_result = ModLoader(Mods[e.Index], c_enabled);  //Loads or Unloads mod by index in mod list based on check state
                    }
                }

                if (!load_result)
                {
                    e.NewValue = CheckState.Unchecked;
                }

                if (!PassReqCheck)
                {
                    e.NewValue = CheckState.Checked;
                }

                AuthorizeCheck = true;

                //Generates an MD5 hash reflecting loaded mods.
                string hash = "";
                foreach (SC2MMData.Mod mod in Mods)
                {
                    if (mod.Enabled)
                    {
                        hash += mod.ModName;
                        hash += mod.Version;
                    }
                }
                if (hash.Length > 0)
                {
                    MD5 md5 = MD5.Create();
                    hash = BitConverter.ToString(md5.ComputeHash(ASCIIEncoding.ASCII.GetBytes(hash))).Replace("-", string.Empty);
                    md5.Dispose();
                    txtMD5.Text = hash;
                }
                else
                {
                    txtMD5.Text = "";
                }


            }
        }

        private bool ConflictCheck(int index)
        {
            string[] delim = { ";" };
            string[] myConflicts = Mods[index].Conflicts.Split(delim, StringSplitOptions.RemoveEmptyEntries);
            foreach (SC2MMData.Mod mod in Mods)
            {
                //MessageBox.Show("Conficts for " + mod.ModName + ": " + mod.Conflicts);
               

                    string[] conflictMods = mod.Conflicts.Split(delim, StringSplitOptions.RemoveEmptyEntries);
                    foreach (string str in conflictMods)
                    {

                        if (Mods[index].ID == str & mod.Enabled)
                            {
                                MessageBox.Show(Mods[index].ModName + "[v" + Mods[index].Version + "]" + "is incompatible with " + mod.ModName + "[v" + mod.Version + "]", "Conflict Detected");
                                return false;
                            }
                        


                    }

                    foreach (string str in myConflicts)
                    {

                        if (mod.ID == str & mod.Enabled)
                        {
                            MessageBox.Show(Mods[index].ModName + "[v" + Mods[index].Version + "]" + "is incompatible with " + mod.ModName + "[v" + mod.Version + "]", "Conflict Detected");
                            return false;
                        }



                    }

                


            }
            return true;
        }

        private bool ReqModCheck(int index)
        {

            List<string> EnabledDependentModIDs = new List<string> { };
            List<string> EnabledDependentModNames = new List<string> { };
            foreach (SC2MMData.Mod mod in Mods)
            {
                if (mod.Enabled & mod.ReqModID.Contains("No Data ") == false)
                {
                    string[] delim = { ";" };
                    string[] RequiredMods = mod.ReqModID.Split(delim, StringSplitOptions.RemoveEmptyEntries);
                    foreach (string str in RequiredMods)
                    {
                        EnabledDependentModIDs.Add(str);
                        EnabledDependentModNames.Add(mod.ModName);
                    }
                }
            }
            int count = 0;
            foreach (string str in EnabledDependentModIDs)
            {
                if (Mods[index].ID == str)
                {
                    MessageBox.Show("This mod is required by " + EnabledDependentModNames[count], "Dependency Alert");
                    return false;
                }
                count++;
            } return true;
        }
        
        

        private bool ModReqCheck(SC2MMData.Mod mod)
        {
            if (mod.ReqModID.Contains("No Data") == false & !SettingsData.LoadingPhase)
            {
                //MessageBox.Show(mod.ReqModID);
                string[] delim = { ";" };
                string[] RequiredMods = mod.ReqModID.Split(delim, StringSplitOptions.RemoveEmptyEntries);
                string[] RequiredModNames = mod.ReqModName.Split(delim, StringSplitOptions.RemoveEmptyEntries);
                string[] RequiredModVers = mod.ReqModVer.Split(delim, StringSplitOptions.RemoveEmptyEntries);

                List<string> Mod_IDs = new List<string> { };
                foreach (SC2MMData.Mod x in Mods)
                {
                    if (x.Enabled)
                    Mod_IDs.Add(x.ID);
                }
                int reqcount = 0;
                foreach (string arg in RequiredMods)
                {
                    foreach (string modid in Mod_IDs)
                    {
                        if (arg == modid)
                        {
                            reqcount++;
                        }
                    }
                }
                if (reqcount != RequiredMods.Length)
                {
                    string c_requiredmodslist = "";

                    for (int x = 0; x < RequiredMods.Length;x++ )
                    {
                        c_requiredmodslist += RequiredModNames[x] + "(" + RequiredMods[x] + ")" + "\n";
                    }
                    MessageBox.Show("Required Mods:\n"+c_requiredmodslist,"Dependency Alert");
                    return false; 
                }
                for (int x = 0; x < RequiredMods.Length; x++)
                {
                    if (RequiredModVers[x].Length != 0)
                    {
                        double version;
                        if (double.TryParse(RequiredModVers[x], out version))
                        {
                            if (Mods[FindIndexByID(RequiredMods[x])].Version < version)
                            {
                                MessageBox.Show(mod.ModName + "[v" + mod.Version + "] requires " + RequiredModNames[x] + "[v" + RequiredModVers[x] + "] or greater.\nYou're using: [v" + Mods[FindIndexByID(RequiredMods[x])].Version.ToString() + "]");
                                return false;
                            }
                        }
                    }
                    
                }

                

            }
            return true;
        }

        private int FindIndexByID(string id_mod) //Returns the index of a mod in the Mods list by its unique ID
        { 
            int id_index = 0;
            foreach (SC2MMData.Mod x in Mods)
            {
                if (id_mod == x.ID)
                {
                    id_index = Mods.IndexOf(x);
                    break;
                }

            }
            return id_index;
        }

        private bool FindDupe(SC2MMData.Mod d_mod) //Compares mod IDs to find duplicates, returns true if found
        {
            foreach (SC2MMData.Mod x in Mods)
            {
                if (d_mod.ID == x.ID || d_mod.DirRef == x.DirRef)
                    return true;
            }
            return false;
        }

        private bool VersionDiff(SC2MMData.Mod vc_mod) //Compares versions between mods with the same ID, queries user for replacement action if different, returns true if 'yes'.
        {
            bool v_result = false;
            foreach (SC2MMData.Mod x in Mods)
            {
                if (vc_mod.ID == x.ID & vc_mod.Version == x.Version)
                {
                    //Do Nothing (Same ID and Version == true duplicate == returns false)
                }
                else if (vc_mod.ID == x.ID) //Same ID and Different version, prompt user for action
                {
                    if (MessageBox.Show
                        (x.ModName + "(v" + x.Version + ") already exists in your library\nDo you wish to replace it with "
                        + vc_mod.ModName + "(v"
                        + vc_mod.Version + ")?", "Replace Previous Mod?", MessageBoxButtons.YesNo) == DialogResult.Yes)
                    {
                        v_result = true; //User replaces existing mod by ID
                    }
                }
                //if (vc_mod.DirRef == x.DirRef & vc_mod.ID != x.ID)
                //{
                //    MessageBox.Show("Duplicate .scd file with unique ID found.\n"+vc_mod.ModName+"("+vc_mod.FileRef+") must be renamed or merged with "+x.ModName+"("+x.FileRef+")", "Duplicate File Warning");
                //}
            }
            return v_result;
        }


        private void btnRemoveMod_Click(object sender, EventArgs e) //Removes selected mod from checkboxlist
        {
            if (chklbModList.SelectedItem != null & ReqModCheck(chklbModList.SelectedIndex))
            {
                int cb_selected = chklbModList.SelectedIndex;
                ModRemove(chklbModList.SelectedIndex);
                if (cb_selected > 0)
                {
                    chklbModList.SetSelected(cb_selected - 1, true);
                }
                else if (cb_selected == 0 & chklbModList.Items.Count > 0)
                {
                    chklbModList.SetSelected(cb_selected, true);
                }
            }
        }

        private void ModRemove(int index) //Accepts index by which a mod is removed
        {
                       
            if (Mods[index].Enabled)
            {
                ModLoader(Mods[index], false);
            }

            ToLog(Mods[index].ModName + "(" + Mods[index].ID + ") v" + Mods[index].Version + " was removed.");
            WipeModFolders(Mods[index]);
            RemoveFromList(index);
            Mods.RemoveAt(index);
            ClearLabels();

        }
        
        private void btnLaunch_Click(object sender, EventArgs e) //Uses steam url function to launch Supreme Commander 2 
        {                                                        //TODO: implement fallthrough from |steam url >> registry >> user specified location| if the preceding fails for some reason
            System.Diagnostics.Process.Start("steam://rungameid/40100");
        }

        /// <summary>
        /// Loads and unloads mods
        /// </summary>
        /// <param name="l_mod">mod to load/unload</param>
        /// <param name="l_state">load if true, unload if false</param>
        private bool ModLoader(SC2MMData.Mod l_mod, bool l_state)
        {
            if (SettingsData.PersistentLoading & SettingsData.LoadingPhase)
            {
                ToLog("Persistent: Skipped Loading Phase");
            }
            else
            {
                string library_modpath = SC2MMData.GetSCDPath() + l_mod.DirRef;
                string gamedata_modpath = SC2MMData.GetSupCom2DataPath();
                List<FileInfo> l_mod_scdfiles = new List<FileInfo> { };
                foreach (string f in Directory.GetFiles(library_modpath))
                {
                    FileInfo file = new FileInfo(f);
                    if (file.Name.Contains(".scd"))
                        l_mod_scdfiles.Add(file);
                }



                if (l_state)
                {
                    bool scd_conflict = false;
                    l_mod.Enabled = true;
                    List<string> conflicts = new List<string> { };
                    foreach (SC2MMData.Mod c_mod in Mods)
                    {
                        foreach (string f in Directory.GetFiles(SC2MMData.GetSCDPath() + c_mod.DirRef))
                        {
                            FileInfo file = new FileInfo(f);
                            if (c_mod.ID != l_mod.ID & c_mod.Enabled)
                            {
                                foreach (FileInfo file_to_be_loaded in l_mod_scdfiles)
                                {
                                    if (file_to_be_loaded.Name == file.Name)
                                    {
                                        conflicts.Add("-----\n" + l_mod.ModName + "[" + file_to_be_loaded.Name + "]\n" + c_mod.ModName + "[" + file.Name + "]");
                                        scd_conflict = true;
                                    }
                                }

                            }
                        }
                    }

                    if (scd_conflict)
                    {
                        string full_conflict_list = "List:\n";
                        foreach (string item in conflicts)
                        {
                            full_conflict_list += item;
                        }
                        MessageBox.Show(full_conflict_list, "Conflict Detected");
                        l_mod.Enabled = false;
                        ToLog("Conflicting SCD list(" + full_conflict_list + ")");
                        return false;
                    }

                    foreach (FileInfo file in l_mod_scdfiles)
                    {
                        string mod_file_destination = gamedata_modpath + file.Name;
                        if (File.Exists(mod_file_destination) & !File.Exists(mod_file_destination + ".original"))
                        {
                            File.Copy(mod_file_destination, mod_file_destination + ".original", true);
                            File.Copy(file.FullName, mod_file_destination, true);
                            ToLog("Replaced original file(" + mod_file_destination + ")");
                        }
                        else
                        {
                            if (File.Exists(file.FullName))
                            {
                                if (File.Exists(mod_file_destination))
                                    File.Delete(mod_file_destination);

                                    File.Move(file.FullName, mod_file_destination);
                                


                                StreamWriter sw = new StreamWriter(file.FullName);
                                sw.Write("Placeholder\nIf you opened this you probably screwed something up.\nClose this window and Windows Explorer before doing anything else.");
                                sw.Close();
                                sw.Dispose();
                                ToLog("Added mod file(" + mod_file_destination + ")");
                                //StatusStrip.Text = "Successfully loaded: " + l_mod.ModName;
                            }
                            else
                            {
                                MessageBox.Show("Missing file from library");
                                ToLog("Couldn't find file(" + mod_file_destination + ")");
                            }
                        }
                    }

                }
                else if (!l_state)
                {
                    l_mod.Enabled = false;
                    foreach (FileInfo file in l_mod_scdfiles)
                    {
                        string mod_file_destination = gamedata_modpath + file.Name;
                        if (File.Exists(mod_file_destination) & File.Exists(mod_file_destination + ".original"))
                        {
                            File.Copy(mod_file_destination + ".original", mod_file_destination, true);
                            File.Delete(mod_file_destination + ".original");
                            ToLog("Restored original file(" + mod_file_destination + ")");
                        }
                        else if (File.Exists(mod_file_destination))
                        {
                            File.Delete(file.FullName);
                            File.Move(mod_file_destination, file.FullName);
                            ToLog("Removed mod file(" + mod_file_destination + ")");
                        }
                    }
                }
            }
            return true;
        }


        //Forces checkbox click to toggle
        bool AuthorizeCheck { get; set; }
        private void chklbModList_MouseDown(object sender, MouseEventArgs e)
        {
            Point loc = chklbModList.PointToClient(Cursor.Position);
            for (int i = 0; i < chklbModList.Items.Count; i++)
            {
                Rectangle rec = chklbModList.GetItemRectangle(i);
                rec.Width = 16;
                if (rec.Contains(loc))
                {
                    AuthorizeCheck = true;
                    bool newValue = !chklbModList.GetItemChecked(i);
                    chklbModList.SetItemChecked(i, newValue);
                    AuthorizeCheck = false;

                    return;
                }
            }
        }

        private void mnuAbout_Click(object sender, EventArgs e)
        {
            frmAbout newAbout = new frmAbout();
            newAbout.StartPosition = FormStartPosition.CenterParent;
            newAbout.ShowDialog();
        }

        private void spltcMain_SplitterMoved(object sender, SplitterEventArgs e)
        {
            chklbModList.Focus();
        }

        private void mnuSettings_Click(object sender, EventArgs e)
        {
            frmSettings newSettings = new frmSettings();

            newSettings.StartPosition = FormStartPosition.CenterParent;
            newSettings.ShowDialog();
            SaveSettings();
        }

        private void mnuForumThread_Click(object sender, EventArgs e)
        {
            if (chklbModList.SelectedItem != null)
            {
                if (Mods[chklbModList.SelectedIndex].ModURL.Contains("No Data for") == false & Mods[chklbModList.SelectedIndex].ModURL != "")
                {
                    System.Diagnostics.Process.Start("http://forums.gaspowered.com/viewtopic.php?f=7&t=" + Mods[chklbModList.SelectedIndex].ModURL);
                }
            }
        }

    }
}

